@@ -34,7 +34,7 @@ import xml.dom.minidom
 from collections import defaultdict, namedtuple
 from datetime import datetime
 from pathlib import Path
-from typing import List, Optional, Tuple, Union
+from typing import Dict, List, Optional, Tuple, Union
 
 # import progress.bar
 import adbutils
@@ -74,6 +74,8 @@ DEBUG = False
 WAIT_FOR_DEVICE_TIMEOUT = int(os.getenv("WAIT_FOR_DEVICE_TIMEOUT", 20))
 
 
+
+
 log_format = '%(color)s[%(levelname)1.1s %(asctime)s %(module)s:%(lineno)d]%(end_color)s [pid:%(process)d] %(message)s'
 formatter = logzero.LogFormatter(fmt=log_format)
 logger = setup_logger("uiautomator2", level=logging.DEBUG, formatter=formatter)
@@ -263,6 +265,7 @@ class _BaseClient(object):
         self._filelock = filelock.FileLock(filelock_path, timeout=200)
 
         self._app_installer: AppInstaller = None
+        self._devWifiOfReset: adbutils.AdbDevice  = None
 
     @property
     def logger(self) -> logging.Logger:
@@ -302,6 +305,9 @@ class _BaseClient(object):
 
     @property
     def _adb_device(self):
+        if self._devWifiOfReset :
+            return self._devWifiOfReset
+
         """ only avaliable when connected with usb """
         assert self._serial, "serial should not empty"
         return adbutils.adb.device(serial=self._serial)
@@ -331,6 +337,8 @@ class _BaseClient(object):
 
         from uiautomator2 import init
         _initer = init.Initer(self._adb_device)
+        if self.settings['reset_atx_listen_addr'] is not None :
+            _initer.set_atx_agent_addr(self.settings['reset_atx_listen_addr'])
         _initer.setup_atx_agent()
 
     def _wait_for_device(self, timeout=None) -> adbutils.AdbDevice:
@@ -363,6 +371,12 @@ class _BaseClient(object):
                     self.logger.debug("adb reconnect error: %s", str(e))
                     time.sleep(1.0)
                     continue
+
+            adbWifiAddr = self.settings['reset_adb_wifi_addr']
+            if (adbWifiAddr is not None) and (self._devWifiOfReset==None) :
+                self._devWifiOfReset = self._connect_by_adbwifi(adbWifiAddr)                
+                return self._devWifiOfReset
+
             try:
                 adb.wait_for(self._serial, timeout=1)
             except adbutils.AdbTimeout:
@@ -371,6 +385,27 @@ class _BaseClient(object):
             return adb.device(self._serial)
         return None
 
+
+
+    def _reset_uiautomator_finish(self) :
+        adbWifiAddr = self.settings['reset_adb_wifi_addr']
+        if (adbWifiAddr is not None) and self._devWifiOfReset :
+            self._disconnect_by_adbwifi(adbWifiAddr)
+            self._devWifiOfReset = None
+
+    def _connect_by_adbwifi(self, addr: str) -> adbutils.AdbDevice :
+        self.logger.info(f"connect by adbwifi:  {addr}") 
+        subprocess.call([adbutils.adb_path(), "connect", addr])
+        try:
+            subprocess.call([adbutils.adb_path(), "-s", addr, "wait-for-device"], timeout=2)
+            return adbutils.adb.device(serial=addr) 
+        except subprocess.TimeoutExpired:
+            return None
+
+    def _disconnect_by_adbwifi(self, addr: str) :
+        self.logger.info(f"disconnect by adbwifi:  ${addr}") 
+        subprocess.call([adbutils.adb_path(), "disconnect", addr])
+
     def _adb_shell(self, cmdargs: Union[list, Tuple[str]], timeout=None):
         """ run command through adb command """
         return self._adb_device.shell(cmdargs, timeout=timeout)
@@ -640,6 +675,7 @@ class _BaseClient(object):
                 launch_test_app=depth > 0)  # uiautomator 2.0
             if ok:
                 self.logger.info("uiautomator back to normal")
+                self._reset_uiautomator_finish()
                 return
 
             output = self._test_run_instrument()
@@ -1957,6 +1993,12 @@ def connect_adb_wifi(addr) -> Device:
     return connect_usb(addr)
 
 
+
+def disconnect_adb_wifi(addr) :
+    subprocess.call([adbutils.adb_path(), "disconnect", addr])
+
+
+
 def connect_usb(serial: Optional[str] = None, init: bool = False) -> Device:
     """
     Args:
@@ -1996,3 +2038,26 @@ def connect_wifi(addr: str) -> Device:
         raise ConnectError("addr is invalid or atx-agent is not running", addr)
     del addr
     return Device(_addr)
+
+
+def pair_by_wifi(addr: str, code: str) -> Optional[str] :
+    """
+    Args:
+        addr (str) pair client address
+        code (str) pair code
+
+    Returns:
+        guid or None
+    """
+    output = subprocess.check_output([adbutils.adb_path(), "pair", addr, code], text=True)
+    if output.startswith('Failed') :
+        logger.error(f"pair by wifi fail: {output}")
+        return None
+    
+    guid = re.findall(r'Successfully .* \[guid=(.+)\]$', output)
+    return guid[0]
+
+def listDeviceSerial() -> List[str] :
+   return list(map(lambda x: x.serial, adbutils.adb.list()))
+
+
